# @Time     : 2021/5/30
# @Project  : f1z1-g
# @IDE      : PyCharm
# @Author   : Angel
# @Email    : 376355670@qq.com

def is_none(value):
    return value is None


def is_not_none(value):
    return value is not None
